import datetime
from telegram import ParseMode, Update
import requests
from telegram.ext import CallbackContext

DIETS = {
    "veggy": ["Vegetarisch", "Vegan"],
    "vegan": ["Vegan"]
}

WEEKDAYS = {
    "montag": 0,
    "dienstag": 1,
    "mittwoch": 2,
    "donnerstag": 3,
    "freitag": 4,
    "samstag": 5,
    "sonntag": 6
}

MENSA_URL = {
    "zentral": "goe1",
    "turm": "goe2",
    "linux": "goe2",
    # "nord": "goe3", nordmensa is currently closed and temporarily replaced by the lunchbox
    "nord": "goe7",
    "italia": "goe4",
    "fasthochschule": "goe5",
    "lunchbox": "goe7"
}

MENSA_NAME = {
    "zentral": "Zentralmensa",
    "nord": "Nordmensa (Lunchbox)",
    "turm": "Turmmensa",
    "linux": "Linuxmensa",
    "italia": "Mensa Italia",
    "fasthochschule": "Bistro Fasthochschule",
    "lunchbox": "Lunchbox (Nordmensa-Ersatz)"
}

HIDE_CATEGORIES_LIGHT = {
    "CampusCurry",
    "natürlich fit",
    "Fitnesscenter",
    "Salatbuffet",
    "Studentenfutter",
    "Süße Versuchung",
    "Süße Spezial Tagesangebot",
    "Vollwert & Co. Stärke",
    "Vollwert & Co. Gemüse",
    "Bio-Beilagen",
    "Dessertbuffet",
    "Last Minute ab 14:30 Uhr",
    ## Nordmensa:
    "Salatbuffet/Pastapoint",
    "Last Minute ab 13:30 Uhr",
    ## Turmmensa:
    "Beilagen",
    "Last Minute ab 14:00Uhr"
}

HIDE_CATEGORIES_FULL = {
    "Last Minute ab 14:30 Uhr",
    "Last Minute ab 13:30 Uhr",
    "Last Minute ab 14:00Uhr"
}

MODES = {
    "light": HIDE_CATEGORIES_LIGHT,
    "full": HIDE_CATEGORIES_FULL
}


def linux_mensa(update: Update, context: CallbackContext):
    context.args = ["linux"]
    mensa(update, context)


def mensa(update: Update, context: CallbackContext):
    which = "zentral"
    filter_categories = MODES["light"]
    diet = [""]
    auto_next_day = True
    today = datetime.datetime.now().date()

    for arg in context.args:
        arg = arg.lower()
        if arg in MENSA_NAME:
            which = arg
        elif arg in MODES:
            filter_categories = MODES[arg]
        elif arg in WEEKDAYS:
            day = WEEKDAYS[arg]
            offset = 0
            if day > today.weekday():
                offset = day - today.weekday()
            elif day < today.weekday():
                offset = (7 - today.weekday() + WEEKDAYS[arg])
            today += datetime.timedelta(days=offset)
            auto_next_day = False
        elif arg in DIETS:
            diet = DIETS[arg]
        else:
            update.message.reply_text("Falscher Aufruf! RTFM und versuchs nochmal.")
            return

    if auto_next_day and datetime.datetime.now().time() > datetime.time(hour=16):
        today += datetime.timedelta(days=1)

    if today.weekday() == 6:
        update.message.reply_text("Sonntags hat die Mensa zu")
        return

    url = "https://app.mensaplan.de/api/11102/de.mensaplan.app.android.goettingen/%s.json" % MENSA_URL[which]

    request = requests.get(url)
    request.encoding = 'utf-8'
    data = request.json()

    today_data = get_data_for_day(today, data)
    if not today_data:
        message = today.strftime("Kein Speiseplan für den %d.%m.%Y vorhanden")
    else:
        message = "Das Essen für %s in der %s \n\n" % (today_data["date"], MENSA_NAME[which])
        for cat in today_data['categories']:
            try:
                label = cat['name']
                meals = cat['meals']
                if label not in filter_categories and (
                        any(filtered in label for filtered in diet) or
                        [item['name'].replace('\xad', '') for item in meals if
                         any(filtered in item['name'].replace('\xad', '') for filtered in diet)]):  # TODO: simplify
                    meal_line = "*%s*\n" % label
                    for meal in meals:
                        meal_line += meal['name'].strip() + "\n"
                    message += meal_line + '\n'
            except:  # TODO: catch correct exception
                print("Could not fetch meal. Skipping.")

    update.message.reply_text(message, ParseMode.MARKDOWN)


def get_data_for_day(day, complete_data):
    for data in complete_data['days']:
        if day.isoformat() == data['iso-date']:
            return data
